@@ -291,6 +291,7 @@ class Presolve
 
    Vec<std::pair<int, int>> presolverStats;
    bool lastRoundReduced{};
+   bool currentRoundReduced{};
    int nunsuccessful{};
    bool rundelayed{};
 
@@ -330,7 +331,8 @@ class Presolve
    bool
    are_applied_tsx_negligible( const Problem<REAL>& problem,
                              const ProblemUpdate<REAL>& probUpdate,
-                             const Statistics& roundStats ) ;
+                             const Statistics& roundStats,
+                             Delegator delegator ) ;
 
    Delegator
    increase_delegator( Delegator delegator );
@@ -407,6 +409,7 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
       num.setFeasTol( REAL{ presolveOptions.feastol } );
       num.setEpsilon( REAL{ presolveOptions.epsilon } );
       num.setHugeVal( REAL{ presolveOptions.hugeval } );
+      num.setUseAbsFeas( presolveOptions.useabsfeas );
 
       Timer timer( stats.presolvetime );
 
@@ -415,12 +418,7 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
       Vec<RowFlags>& rflags = constraintMatrix.getRowFlags();
       const Vec<int>& rowsize = constraintMatrix.getRowSizes();
 
-      msg.info( "\nstarting presolve of problem {}:\n", problem.getName() );
-      msg.info( "  rows:     {}\n", problem.getNRows() );
-      msg.info( "  columns:  {}\n", problem.getNCols() );
-      msg.info( "  int. columns:  {}\n", problem.getNumIntegralCols() );
-      msg.info( "  cont. columns:  {}\n", problem.getNumContinuousCols() );
-      msg.info( "  nonzeros: {}\n\n", problem.getConstraintMatrix().getNnz() );
+
 
       PresolveResult<REAL> result;
 
@@ -443,10 +441,11 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
          {
             msg.error(
                 "Please turn off the presolvers substitution and sparsify and "
-                "componentsdetection to use dual postsolving\n" );
-            return result;
+                "componentsdetection to use dual-postsolving\n" );
+            msg.error("Continuing without dual-presolve\n");
          }
       }
+
       if(presolveOptions.verification_with_VeriPB &&
           problem.test_problem_type( ProblemFlag::kBinary ))
       {
@@ -455,6 +454,13 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
          certificate_interface->print_header();
       }
 
+      msg.info( "\nstarting presolve of problem {} with dual-postsolve {}activated\n", problem.getName(), result.postsolve.postsolveType == PostsolveType::kFull?"":"de-" );
+      msg.info( "  rows:     {}\n", problem.getNRows() );
+      msg.info( "  columns:  {}\n", problem.getNCols() );
+      msg.info( "  int. columns:  {}\n", problem.getNumIntegralCols() );
+      msg.info( "  cont. columns:  {}\n", problem.getNumContinuousCols() );
+      msg.info( "  nonzeros: {}\n\n", problem.getConstraintMatrix().getNnz() );
+
       result.status = PresolveStatus::kUnchanged;
 
       std::stable_sort( presolvers.begin(), presolvers.end(),
@@ -491,9 +497,6 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
 
       reductions.resize( presolvers.size() );
       results.resize( presolvers.size() );
-
-      round_to_evaluate = Delegator::kFast;
-
       presolverStats.resize( presolvers.size(), std::pair<int, int>( 0, 0 ) );
 
       ProblemUpdate<REAL> probUpdate( problem, result.postsolve, stats,
@@ -509,21 +512,7 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
          }
       }
 
-      result.status = probUpdate.trivialPresolve();
-
-      if( result.status == PresolveStatus::kInfeasible ||
-          result.status == PresolveStatus::kUnbndOrInfeas ||
-          result.status == PresolveStatus::kUnbounded )
-      {
-         probUpdate.getCertificateInterface()->infeasible();
-         return result;
-      }
-      printRoundStats( false, "Trivial" );
       round_to_evaluate = Delegator::kFast;
-
-      finishRound( probUpdate );
-      ++stats.nrounds;
-
       nunsuccessful = 0;
       rundelayed = true;
       for( int i = 0; i < npresolvers; ++i )
@@ -538,12 +527,35 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
       Statistics last_rounds_stats = stats;
       do
       {
-         bool was_executed_sequential = false;
-         // if problem is trivial abort here
-         if( probUpdate.getNActiveCols() == 0 ||
-             probUpdate.getNActiveRows() == 0 )
+         if( round_to_evaluate == Delegator::kFast )
+         {
+            if( presolveOptions.maxrounds != -1 && presolveOptions.maxrounds <= stats.nrounds )
+            {
+               msg.info("Maximum round {} reached. Finishing...\n", presolveOptions.maxrounds);
+               round_to_evaluate = Delegator::kAbort;
+            }
+
+            result.status = probUpdate.trivialPresolve();
+
+            if( stats.nrounds == 0 )
+               printRoundStats( false, "Trivial" );
+
+            if( is_status_infeasible_or_unbounded( result.status ) )
+               return result;
+
+            if( probUpdate.getNActiveCols() == 0 || probUpdate.getNActiveRows() == 0 )
+               round_to_evaluate = Delegator::kAbort;
+
+            lastRoundReduced = currentRoundReduced;
+            currentRoundReduced = false;
+            ++stats.nrounds;
+         }
+
+         if( round_to_evaluate == Delegator::kAbort )
             break;
 
+         bool was_executed_sequential = false;
+
          switch( round_to_evaluate )
          {
          case Delegator::kFast:
@@ -562,29 +574,15 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
             assert( false );
          }
 
-         result.status =
-             evaluate_and_apply( timer, problem, result, probUpdate,
-                                 last_rounds_stats, was_executed_sequential );
+         result.status = evaluate_and_apply( timer, problem, result, probUpdate,last_rounds_stats,
+               was_executed_sequential );
+
          if( is_status_infeasible_or_unbounded( result.status ) )
             return result;
+
          last_rounds_stats = stats;
-
-      } while( round_to_evaluate != Delegator::kAbort );
-
-      if( stats.ntsxapplied > 0 || stats.nboundchgs > 0 ||
-          stats.ncoefchgs > 0 || stats.ndeletedcols > 0 ||
-          stats.ndeletedrows > 0 || stats.nsidechgs > 0 )
-      {
-         result.status = probUpdate.trivialPresolve();
-
-         if( result.status == PresolveStatus::kInfeasible ||
-             result.status == PresolveStatus::kUnbndOrInfeas ||
-             result.status == PresolveStatus::kUnbounded )
-            return result;
-
-         probUpdate.clearStates();
-         probUpdate.check_and_compress();
       }
+      while( true );
 
       if(probUpdate.getProblem().getNCols() > 0)
       {
@@ -751,18 +749,14 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
                          nremoved, nnewfreevars );
          }
 
-         bool detectComponents = presolveOptions.componentsmaxint != -1;
-
-         if( !lpSolverFactory && problem.getNumContinuousCols() != 0 )
-            detectComponents = false;
-
-         if( !(mipSolverFactory || satSolverFactory) && problem.getNumIntegralCols() != 0 )
-            detectComponents = false;
-
-         if( problem.getNCols() == 0 )
-            detectComponents = false;
-
-         if( detectComponents  && probUpdate.getNActiveCols() > 0 )
+         // detect disconnected components
+         if( presolveOptions.componentsmaxint != -1
+            && presolveOptions.dualreds == 2
+            && probUpdate.getNActiveCols() >= 1
+            && ( mipSolverFactory != nullptr
+               || ( ( lpSolverFactory != nullptr || problem.getNumContinuousCols() == 0 )
+                  //TODO: support SAT solver
+                  && ( /* satSolverFactory != nullptr || */ problem.getNumIntegralCols() == 0 ) ) ) )
          {
             assert( problem.getNCols() != 0 && problem.getNRows() != 0 );
             Components components;
@@ -808,7 +802,8 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
 
 #endif
                       {
-                         if( compInfo[i].nintegral == 0 )
+                         if( lpSolverFactory != nullptr
+                            && compInfo[i].nintegral == 0 )
                          {
                             std::unique_ptr<SolverInterface<REAL>> solver =
                                 lpSolverFactory->newSolver(
@@ -842,10 +837,16 @@ Presolve<REAL>::apply( Problem<REAL>& problem, bool store_dual_postsolve )
                                       true;
                             }
                          }
-                         else if( compInfo[i].nintegral <=
-                                  presolveOptions.componentsmaxint )
+                         //TODO: call SAT solver
+                         // else if( satSolverFactory != nullptr
+                         //    && compInfo[i].ncontinuous == 0
+                         //    && compInfo[i].nintegral <= presolveOptions.componentsmaxint
+                         //    && <pure binary component>
+                         // {
+                         // }
+                         else if( mipSolverFactory != nullptr
+                            && compInfo[i].nintegral <= presolveOptions.componentsmaxint )
                          {
-                            //TODO: add satsolverfactory
                             std::unique_ptr<SolverInterface<REAL>> solver =
                                 mipSolverFactory->newSolver(
                                     VerbosityLevel::kQuiet );
@@ -1003,14 +1004,6 @@ Presolve<REAL>::run_presolvers( const Problem<REAL>& problem,
          if( problem.getNRows() == 0 || problem.getNCols() == 0 )
             return;
       }
-      PresolveStatus status = probUpdate.trivialPresolve();
-      if( is_status_infeasible_or_unbounded( status ) )
-      {
-         results[presolver_2_run.first] = status;
-         return;
-      }
-      probUpdate.clearStates();
-      probUpdate.check_and_compress();
    }
 #ifdef PAPILO_TBB
    else
@@ -1081,12 +1074,6 @@ Presolve<REAL>::evaluate_and_apply( const Timer& timer, Problem<REAL>& problem,
                                     const Statistics& oldstats,
                                     bool run_sequential )
 {
-   if( round_to_evaluate == Delegator::kFast )
-   {
-      probUpdate.clearChangeInfo();
-      lastRoundReduced = false;
-   }
-
    result.status = evaluateResults();
    switch( result.status )
    {
@@ -1316,7 +1303,6 @@ Presolve<REAL>::handle_case_exceeded( Delegator& next_round )
             p->setDelayed( false );
          rundelayed = true;
       }
-      ++stats.nrounds;
       return Delegator::kFast;
    }
    printRoundStats( !lastRoundReduced, get_round_type( next_round ) );
@@ -1335,11 +1321,32 @@ template <typename REAL>
 bool
 Presolve<REAL>::are_applied_tsx_negligible( const Problem<REAL>& problem,
                                           const ProblemUpdate<REAL>& probUpdate,
-                                          const Statistics& roundStats )
+                                          const Statistics& roundStats,
+                                          Delegator delegator )
 {
-   double abort_factor = problem.getNumIntegralCols() == 0
+   double abort_factor = 0;
+   switch ( delegator )
+   {
+   case Delegator::kFast:
+      abort_factor = problem.getNumIntegralCols() == 0
+                             ? presolveOptions.lpabortfacfast
+                             : presolveOptions.abortfacfast;
+      break;
+   case Delegator::kMedium:
+      abort_factor = problem.getNumIntegralCols() == 0
+                             ? presolveOptions.lpabortfacmedium
+                             : presolveOptions.abortfacmedium;
+      break;
+   case Delegator::kExhaustive:
+      abort_factor = problem.getNumIntegralCols() == 0
                              ? presolveOptions.lpabortfac
                              : presolveOptions.abortfac;
+      break;
+  case Delegator::kAbort:
+  case Delegator::kExceeded:
+      assert(false);
+   }
+   
    if( roundStats.ndeletedcols == 0 && roundStats.ndeletedrows == 0 &&
        roundStats.ncoefchgs == 0 && presolveOptions.max_consecutive_rounds_of_only_bound_changes >= 0 )
    {
@@ -1367,10 +1374,10 @@ Presolve<REAL>::increase_round_if_last_run_was_not_successfull(
    Delegator next_round;
    if( !unchanged )
    {
-      if( are_applied_tsx_negligible( problem, probUpdate, roundStats ) )
+      if( are_applied_tsx_negligible( problem, probUpdate, roundStats, round_to_evaluate ) )
       {
-         lastRoundReduced =
-             lastRoundReduced || roundStats.nsidechgs > 0 ||
+         currentRoundReduced =
+             currentRoundReduced || roundStats.nsidechgs > 0 ||
              roundStats.nboundchgs > 0 || roundStats.ndeletedcols > 0 ||
              roundStats.ndeletedrows > 0 || roundStats.ncoefchgs > 0;
          next_round = increase_delegator( round_to_evaluate );
@@ -1378,10 +1385,9 @@ Presolve<REAL>::increase_round_if_last_run_was_not_successfull(
       else
       {
          printRoundStats( false, get_round_type( round_to_evaluate ) );
-         lastRoundReduced = true;
-         next_round = Delegator::kFast;
          nunsuccessful = 0;
-         ++stats.nrounds;
+         currentRoundReduced = true;
+         next_round = Delegator::kFast;
       }
    }
    else
