@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -45,6 +44,7 @@
 
 #include <fstream>
 
+#ifdef PAPILO_SERIALIZATION_AVAILABLE
 #include <boost/archive/text_iarchive.hpp>
 #include <boost/archive/text_oarchive.hpp>
 #include <boost/archive/tmpdir.hpp>
@@ -52,6 +52,7 @@
 #include <boost/serialization/list.hpp>
 #include <boost/serialization/utility.hpp>
 #include <boost/serialization/vector.hpp>
+#endif
 
 namespace papilo
 {
@@ -272,7 +273,10 @@ Postsolve<REAL>::undo( const Solution<REAL>& reducedSolution,
          sumcols.add( -side );
 
          assert( colCoef != 0.0 );
-         originalSolution.primal[col] = ( -sumcols.get() ) / colCoef;
+         if( num.isEq( sumcols.get(), 0 ) )
+            originalSolution.primal[col] = 0;
+         else
+            originalSolution.primal[col] = ( -sumcols.get() ) / colCoef;
          break;
       }
       case ReductionType::kSubstitutedColWithDual:
@@ -464,6 +468,7 @@ Postsolve<REAL>::copy_from_reduced_to_original(
    originalSolution.primal.clear();
    originalSolution.primal.resize( postsolveStorage.nColsOriginal );
 
+   assert(reducedSolution.primal.size() == postsolveStorage.origcol_mapping.size());
    int reduced_columns = (int)reducedSolution.primal.size();
    for( int k = 0; k < reduced_columns; ++k )
       originalSolution.primal[postsolveStorage.origcol_mapping[k]] =
@@ -491,22 +496,28 @@ Postsolve<REAL>::copy_from_reduced_to_original(
          originalSolution.reducedCosts[postsolveStorage.origcol_mapping[k]] =
              reducedSolution.reducedCosts[k];
 
-      assert( (int) reducedSolution.varBasisStatus.size() == reduced_columns );
-      originalSolution.varBasisStatus.clear();
-      originalSolution.varBasisStatus.resize( postsolveStorage.nColsOriginal,
-                                              VarBasisStatus::UNDEFINED );
-      for( int k = 0; k < reduced_columns; k++ )
-         originalSolution.varBasisStatus[postsolveStorage.origcol_mapping[k]] =
-             reducedSolution.varBasisStatus[k];
+      if( reducedSolution.basisAvailabe )
+      {
+         assert( (int)reducedSolution.varBasisStatus.size() ==
+                 reduced_columns );
+         originalSolution.varBasisStatus.clear();
+         originalSolution.varBasisStatus.resize( postsolveStorage.nColsOriginal,
+                                                 VarBasisStatus::UNDEFINED );
+         for( int k = 0; k < reduced_columns; k++ )
+            originalSolution
+                .varBasisStatus[postsolveStorage.origcol_mapping[k]] =
+                reducedSolution.varBasisStatus[k];
 
-      assert( (int) reducedSolution.rowBasisStatus.size() == reduced_rows );
+         assert( (int)reducedSolution.rowBasisStatus.size() == reduced_rows );
 
-      originalSolution.rowBasisStatus.clear();
-      originalSolution.rowBasisStatus.resize( postsolveStorage.nRowsOriginal,
-                                              VarBasisStatus::UNDEFINED );
-      for( int k = 0; k < reduced_rows; k++ )
-         originalSolution.rowBasisStatus[postsolveStorage.origrow_mapping[k]] =
-             reducedSolution.rowBasisStatus[k];
+         originalSolution.rowBasisStatus.clear();
+         originalSolution.rowBasisStatus.resize( postsolveStorage.nRowsOriginal,
+                                                 VarBasisStatus::UNDEFINED );
+         for( int k = 0; k < reduced_rows; k++ )
+            originalSolution
+                .rowBasisStatus[postsolveStorage.origrow_mapping[k]] =
+                reducedSolution.rowBasisStatus[k];
+      }
    }
 }
 
@@ -561,6 +572,10 @@ Postsolve<REAL>::apply_fix_infinity_variable_in_original_solution(
    bool isNegativeInfinity = values[first] < 0;
    int* row_indices = new int[number_rows];
    REAL* col_coefficents = new REAL[number_rows];
+
+   if( number_rows == 0 && bound == std::numeric_limits<int64_t>::max() )
+      solution = 0;
+
    if( isNegativeInfinity )
    {
       while( row_counter < number_rows )
@@ -650,16 +665,20 @@ Postsolve<REAL>::apply_fix_infinity_variable_in_original_solution(
          sum.add( -col_coefficents[k] * originalSolution.dual[row_indices[k]] );
       originalSolution.reducedCosts[col] = sum.get();
 
+      bool is_bound_infinity = bound == std::numeric_limits<int64_t>::max();
+
       // store the bounds of the variable
       if( isNegativeInfinity )
-         stored_bounds.set_bounds_of_variable( col, true, false, 0, bound );
+         stored_bounds.set_bounds_of_variable( col, true, is_bound_infinity, 0, bound );
       else
-         stored_bounds.set_bounds_of_variable( col, false, true, bound, 0 );
+         stored_bounds.set_bounds_of_variable( col, is_bound_infinity, true, bound, 0 );
 
       // set the basis depending on the status
       if( originalSolution.basisAvailabe )
       {
-         if( num.isEq( solution, bound ) )
+         if( number_rows == 0 && bound == std::numeric_limits<int64_t>::max() )
+            originalSolution.varBasisStatus[col] = VarBasisStatus::ZERO;
+         else if( num.isEq( solution, bound ) )
             if( isNegativeInfinity )
                originalSolution.varBasisStatus[col] = VarBasisStatus::ON_UPPER;
             else
@@ -698,7 +717,10 @@ Postsolve<REAL>::apply_substituted_column_to_original_solution(
    }
    sumcols.add( -lhs );
    assert( colCoef != 0.0 );
-   originalSolution.primal[col] = ( -sumcols.get() ) / colCoef;
+   if( num.isEq( sumcols.get(), 0) )
+      originalSolution.primal[col] = 0;
+   else
+      originalSolution.primal[col] = ( -sumcols.get() ) / colCoef;
 
    assert( ( originalSolution.type == SolutionType::kPrimalDual &&
              values[first + 3 + row_length] > 0 ) ||
@@ -856,7 +878,7 @@ Postsolve<REAL>::apply_row_bound_change_to_original_solution(
          {
             if( isLhs )
             {
-               if( num.isLT( factor, 0 ) )
+               if( factor < 0 )
                   originalSolution.rowBasisStatus[deleted_row] =
                       VarBasisStatus::ON_UPPER;
                else
@@ -866,7 +888,7 @@ Postsolve<REAL>::apply_row_bound_change_to_original_solution(
             }
             else
             {
-               if( num.isLT( factor, 0 ) )
+               if( factor < 0 )
                   originalSolution.rowBasisStatus[deleted_row] =
                       VarBasisStatus::ON_LOWER;
                else
@@ -889,8 +911,18 @@ Postsolve<REAL>::apply_row_bound_change_to_original_solution(
             }
             else
             {
-               originalSolution.rowBasisStatus[deleted_row] =
-                   originalSolution.rowBasisStatus[row];
+               if( factor > 0)
+               {
+                  originalSolution.rowBasisStatus[ deleted_row ] =
+                        originalSolution.rowBasisStatus[ row ];
+               }
+               else
+               {
+                  if( originalSolution.rowBasisStatus[ row ] == VarBasisStatus::ON_LOWER )
+                     originalSolution.rowBasisStatus[ deleted_row ] = VarBasisStatus::ON_UPPER;
+                  else if( originalSolution.rowBasisStatus[ row ] == VarBasisStatus::ON_UPPER )
+                     originalSolution.rowBasisStatus[ deleted_row ] = VarBasisStatus::ON_LOWER;
+               }
                originalSolution.rowBasisStatus[row] = VarBasisStatus::BASIC;
             }
          }
@@ -942,9 +974,9 @@ Postsolve<REAL>::apply_var_bound_change_forced_by_column_in_original_solution(
 
    const REAL reduced_costs = originalSolution.reducedCosts[col];
    bool changes_neg_reduced_costs =
-       ! isLowerBound && num.isLT( reduced_costs, 0 );
+       ! isLowerBound && num.isFeasLT( reduced_costs, 0 );
    bool changes_pos_reduced_costs =
-       isLowerBound && num.isGT( reduced_costs, 0 );
+       isLowerBound && num.isFeasGT( reduced_costs, 0 );
 
    int variables_removed_from_basis = 0;
 
@@ -952,7 +984,7 @@ Postsolve<REAL>::apply_var_bound_change_forced_by_column_in_original_solution(
    if( num.isFeasEq( new_value, originalSolution.primal[col] ) &&
        ( changes_neg_reduced_costs || changes_pos_reduced_costs ) )
    {
-      assert( ! num.isZero( reduced_costs ) );
+      assert( ! num.isFeasZero( reduced_costs ) );
       SavedRow<REAL> saved_row{
           num, i, types, start, indices, values, originalSolution.primal };
       int row = saved_row.getRow();
@@ -1107,18 +1139,21 @@ Postsolve<REAL>::apply_parallel_col_to_original_solution(
       assert( !( col2boundFlags & IS_LBINF ) );
       assert( !( col2boundFlags & IS_UBINF ) );
       assert( col2boundFlags & IS_INTEGRAL );
+      assert( num.isIntegral(col2scale) );
 
-      col1val = col1lb;
+      col1val = num.max(col1lb, num.min(col1ub, 0));
+      col2val = solval - col2scale * col1val;
 
-      while( num.isFeasLE( col1val, col1ub ) )
+      // recompute feasible values next to the closest bound
+      if( num.isFeasLT( col2val, col2lb ) )
       {
-         col2val = solval - col1val * col2scale;
-
-         if( num.isFeasIntegral( col2val ) && num.isFeasGE( col2val, col2lb ) &&
-             num.isFeasLE( col2val, col2ub ) )
-            break;
-
-         col1val += 1;
+         col1val = num.round((1 - abs(num.round(col2scale)) + 2 * (solval - col2lb)) / (2 * col2scale));
+         col2val = solval - col2scale * col1val;
+      }
+      else if( num.isFeasGT( col2val, col2ub ) )
+      {
+         col1val = num.round((abs(num.round(col2scale)) - 1 + 2 * (solval - col2ub)) / (2 * col2scale));
+         col2val = solval - col2scale * col1val;
       }
    }
    else
