@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -62,6 +61,7 @@ class Probing : public PresolveMethod<REAL>
       this->setType( PresolverType::kIntegralCols );
    }
 
+
    void
    compress( const Vec<int>& rowmap, const Vec<int>& colmap ) override
    {
@@ -101,7 +101,7 @@ class Probing : public PresolveMethod<REAL>
 
       paramSet.addParameter( "probing.maxbadgesize",
                              "maximal number of probing candidates probed in "
-                             "a single badge of candidates",
+                             "a single badge of candidates (-1, 0: unlimited)",
                              max_badge_size, DEFAULT_MAX_BADGE_SIZE );
 
       paramSet.addParameter(
@@ -113,13 +113,15 @@ class Probing : public PresolveMethod<REAL>
 
    PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility) override;
 
    bool
    isBinaryVariable( REAL upper_bound, REAL lower_bound, int column_size,
                      const Flags<ColFlag>& colFlag ) const;
 
+
    void
    set_max_badge_size( int val);
 
@@ -136,7 +138,7 @@ PresolveStatus
 Probing<REAL>::execute( const Problem<REAL>& problem,
                         const ProblemUpdate<REAL>& problemUpdate,
                         const Num<REAL>& num, Reductions<REAL>& reductions,
-                        const Timer& timer)
+                        const Timer& timer, int& reason_of_infeasibility)
 {
    if( problem.getNumIntegralCols() == 0 )
       return PresolveStatus::kUnchanged;
@@ -350,6 +352,7 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
    boundChanges.reserve( ncols );
 
    std::atomic_bool infeasible{ false };
+   std::atomic_int infeasible_variable {-1};
 
    // use tbb combinable so that each thread will copy the activities and
    // bounds at most once
@@ -416,6 +419,7 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
                    if( globalInfeasible )
                    {
                       infeasible.store( true, std::memory_order_relaxed );
+                      infeasible_variable.store(col);
                       break;
                    }
                 }
@@ -431,7 +435,10 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
          return PresolveStatus::kUnchanged;
 
       if( infeasible.load( std::memory_order_relaxed ) )
+      {
+         reason_of_infeasibility = infeasible_variable.load(std::memory_order_relaxed);
          return PresolveStatus::kInfeasible;
+      }
 
       int64_t amountofwork = 0;
       int nfixings = 0;
@@ -463,8 +470,7 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
             {
                // found new bound change
                boundChanges.emplace_back( boundChg );
-               boundPos[2 * boundChg.col + boundChg.upper] =
-                   boundChanges.size();
+               boundPos[2 * boundChg.col + boundChg.upper] = boundChanges.size();
 
                // check if column is now fixed
                if( ( boundChg.upper &&
@@ -478,9 +484,7 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
             else
             {
                // already changed that bound
-               ProbingBoundChg<REAL>& otherBoundChg =
-                   boundChanges[boundPos[2 * boundChg.col + boundChg.upper] -
-                                1];
+               ProbingBoundChg<REAL>& otherBoundChg = boundChanges[boundPos[2 * boundChg.col + boundChg.upper] - 1];
 
                if( boundChg.upper && boundChg.bound < otherBoundChg.bound )
                {
@@ -490,6 +494,13 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
                   // check if column is now fixed
                   if( boundChg.bound == lower_bounds[boundChg.col] )
                      ++nfixings;
+
+                  if(problemUpdate.getPresolveOptions().verification_with_VeriPB)
+                  {
+                     if(boundChg.probing_col == -1 )
+                        otherBoundChg.probing_col = -1;
+                  }
+
                }
                else if( !boundChg.upper &&
                         boundChg.bound > otherBoundChg.bound )
@@ -551,18 +562,22 @@ Probing<REAL>::execute( const Problem<REAL>& problem,
 
    if( !boundChanges.empty() )
    {
-      pdqsort(
-          boundChanges.begin(), boundChanges.end(),
-          []( const ProbingBoundChg<REAL>& a, const ProbingBoundChg<REAL>& b ) {
-             return ( a.col << 1 | a.upper ) < ( b.col << 1 | b.upper );
-          } );
 
       for( const ProbingBoundChg<REAL>& boundChg : boundChanges )
       {
          if( boundChg.upper )
+         {
+            if(problemUpdate.getPresolveOptions().verification_with_VeriPB && boundChg.probing_col!= -1)
+               reductions.reason_probing_upper_bound_change(boundChg.probing_col, boundChg.col);
             reductions.changeColUB( boundChg.col, boundChg.bound );
+         }
          else
+         {
+            if(problemUpdate.getPresolveOptions().verification_with_VeriPB && boundChg.probing_col!= -1)
+               reductions.reason_probing_lower_bound_change(boundChg.probing_col, boundChg.col);
             reductions.changeColLB( boundChg.col, boundChg.bound );
+
+         }
       }
 
       result = PresolveStatus::kReduced;
